/*
 * IDrawStr.c
 *
 * Image library
 *
 * Description:
 *	Portable routines to manipulate raster images.
 *
 * History:
 *	18-May-98	Craig Knudsen	cknudsen@radix.net
 *			Added support for anti-aliasing fonts by using a
 *			font 2X bigger than we need.
 *	20-May-96	Craig Knudsen	cknudsen@radix.net
 *			Created
 *
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <memory.h>

#include "Ilib.h"
#include "IlibP.h"
#include "IFontBDF.h"


#define SPACES_PER_TAB		8



static void anti_alias ( bg, fg, level, ret )
IColorP *bg;
IColorP *fg;
int level;	/* 0-4 */
IColorP *ret;
{
  unsigned int temp;

  temp = ( (unsigned int)bg->red * ( 4 - level ) + 
    (unsigned int)fg->red * ( level ) ) / 4;
  ret->red = (unsigned char) temp;

  temp = ( (unsigned int)bg->green * ( 4 - level ) + 
    (unsigned int)fg->green * ( level ) ) / 4;
  ret->green = (unsigned char) temp;

  temp = ( (unsigned int)bg->blue * ( 4 - level ) + 
    (unsigned int)fg->blue * ( level ) ) / 4;
  ret->blue = (unsigned char) temp;

}


IError IDrawString ( image, gc, x, y, text, len )
IImage image;
IGC gc;
int x;
int y;
char *text;
unsigned int len;
{
  IGCP *gcp = (IGCP *)gc;
  IImageP *imagep = (IImageP *)image;
  unsigned int *bitdata;
  unsigned int height, width, actual_width, size, font_height;
  int xoffset, yoffset, charx, chary;
  char ch[256], *ptr;
  int loop, loop2, loop3;
  IError ret;
  int myx, myy;
  int char_num = 0;
  int level;
  IColorP bg; /* for anti-aliasing */
  IColorP aac; /* for anti-aliasing */

  if ( ! gcp )
    return ( IInvalidGC );
  if ( gcp->magic != IMAGIC_GC )
    return ( IInvalidGC );

  if ( ! imagep )
    return ( IInvalidImage );
  if ( imagep->magic != IMAGIC_IMAGE )
    return ( IInvalidImage );

  if ( ! gcp->font )
    return ( INoFontSet );

  charx = x;
  chary = y;

  IFontSize ( (IFont)gcp->font, &font_height );

  for ( ptr = text, loop = 0; loop < len; loop++, ptr++ ) {
    if ( *ptr == '\012' ) {
      charx = x;
      chary += ( gcp->antialiased ? ( font_height / 2 ) : font_height );
      char_num = 0;
      continue;
    }
    else if ( *ptr == '\t' ) {
      ret = IFontBDFGetChar ( gcp->font->name, ch, &bitdata, &width, &height,
        &actual_width, &size, &xoffset, &yoffset );
      if ( gcp->antialiased )
        charx += ( ( 8 - ( char_num % 8 ) ) * actual_width / 2 );
      else
        charx += ( 8 - ( char_num % 8 ) ) * actual_width;
      continue;
    }
    else if ( *ptr != '\033' ) {
      ch[0] = *ptr;
      ch[1] = '\0';
    }
    else {
      loop2 = 0;
      ptr++;
      while ( *ptr != ';' && loop < len && loop2 < 256 ) {
        ch[loop2] = *ptr;
        ptr++;
        loop++;
        loop2++;
      }
      ch[loop2] = '\0';
      if ( *ptr != ';' ) {
        return ( IInvalidEscapeSequence );
      }
    }
    ret = IFontBDFGetChar ( gcp->font->name, ch, &bitdata, &width, &height,
      &actual_width, &size, &xoffset, &yoffset );
    if ( ! ret ) {
      if ( gcp->antialiased ) {
        for ( loop3 = 0; loop3 < height; loop3 += 2 ) {
          myy = chary - ( height + yoffset ) / 2 + loop3 / 2;
          for ( loop2 = 0; loop2 < width; loop2 += 2 ) {
            level = 0;
            if ( bitdata[loop3 * width + loop2] )
              level++;
            if ( loop3 + 1 < height && bitdata[(loop3+1) * width + loop2] )
              level++;
            if ( loop2 + 1 < width && bitdata[loop3 * width + (loop2+1)] )
              level++;
            if ( loop2 + 1 < width && loop3 + 1 < height &&
              bitdata[(loop3+1) * width + (loop2+1)] )
              level++;
            if ( level ) {
              _IGetPointColor ( imagep, myx, myy, bg );
              anti_alias ( &bg, gcp->foreground, level, &aac );
              myx = charx + ( xoffset + loop2 ) / 2;
              _ISetPointRGB ( imagep, myx, myy, aac.red, aac.green, aac.blue );
            }
          }
        }
        charx += actual_width / 2;
      } else {
        for ( loop3 = 0; loop3 < height; loop3++ ) {
          myy = chary - ( height + yoffset ) + loop3;
          for ( loop2 = 0; loop2 < width; loop2++ ) {
            if ( bitdata[loop3 * width + loop2] ) {
              myx = charx + xoffset + loop2;
              _ISetPoint ( imagep, gcp, myx, myy );
            }
          }
        }
        charx += actual_width;
      }
      char_num++;
    }
  }

  return ( INoError );
}



