/*
 * IXPM.c
 *
 * Image library
 *
 * Description:
 *	Portable routines to manipulate raster images.
 *
 * History:
 *	20-May-96	Craig Knudsen	cknudsen@radix.net
 *			Created
 *
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <memory.h>

#include "Ilib.h"
#include "IlibP.h"

#define colors_match(color,r,g,b) \
 (color->red == red && color->green == green && color->blue == blue )

#define MAX_COLORMAP_SIZE	(256*256)


IError _IWriteXPM ( fp, image, options )
FILE *fp;
IImageP *image;
IOptions options;
{
  int r, c;
  unsigned char *ptr;
  unsigned int red, green, blue;
  IColorP *colormap[MAX_COLORMAP_SIZE];
  int num_colors = 0;
  int loop, loop2;
  int offset;
  unsigned short *data;
  int color_found, chars_per_pixel;
  static char *xpm_chars = \
    ".#abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

  data = (unsigned short *) calloc ( image->width * image->height, 
    sizeof ( unsigned short ) );
  memset ( data, '\0', image->width * image->height * sizeof ( short ) );

  for ( r = 0; r < image->height; r++ ) {
    for ( c = 0; c < image->width; c++ ) {
      offset = ( r * image->width ) + c;
      if ( image->greyscale ) {
        ptr = image->data + ( r * image->width ) + c;
        red = green = blue = (unsigned int) *ptr;
      }
      else {
        ptr = image->data + ( r * image->width * 3 ) + ( c * 3 );
        red = (unsigned int) *ptr;
        green = (unsigned int) *( ptr + 1 );
        blue = (unsigned int) *( ptr + 2 );
      }
      color_found = 0;
      for ( loop = 0; loop < num_colors; loop++ ) {
        if ( colors_match ( colormap[loop], red, green, blue ) ) {
          data[offset] = loop;
          color_found = 1;
          break;
        }
      }
      if ( ! color_found ) {
        if ( num_colors < MAX_COLORMAP_SIZE ) {
          colormap[num_colors] = (IColorP *) malloc ( sizeof ( IColorP ) );
          colormap[num_colors]->red = red;
          colormap[num_colors]->green = green;
          colormap[num_colors]->blue = blue;
          data[offset] = num_colors;
          num_colors++;
        }
      }
    }
  }

  fprintf ( fp, "/* XPM */\n" );
  if ( image->comments )
    fprintf ( fp, "/* %s */\n", image->comments );
  fprintf ( fp, "static char *image[] = {\n" );
  fprintf ( fp, "/* width height num_colors chars_per_pixel */\n" );
  if ( num_colors > strlen ( xpm_chars ) )
    chars_per_pixel = 2;
  else
    chars_per_pixel = 1;
  fprintf ( fp, "\"%d %d %d %d\",\n", image->width, image->height,
    num_colors, chars_per_pixel );

  fprintf ( fp, "/* colors */\n" );
  for ( loop = 0; loop < num_colors; loop++ ) {
    switch ( chars_per_pixel ) {
      case 1:
        fprintf ( fp, "\"%c c #%02X%02X%02X\",\n",
          xpm_chars[loop],
          colormap[loop]->red,
          colormap[loop]->green,
          colormap[loop]->blue );
        break;
      case 2:
        fprintf ( fp, "\"%c%c c #%02X%02X%02X\",\n",
          xpm_chars[loop/strlen(xpm_chars)],
          xpm_chars[loop%strlen(xpm_chars)],
          colormap[loop]->red,
          colormap[loop]->green,
          colormap[loop]->blue );
        break;
    }
  }

  fprintf ( fp, "/* pixels */\n" );
  for ( loop = 0; loop < image->height; loop++ ) {
    fprintf ( fp, "\"" );
    for ( loop2 = 0; loop2 < image->width; loop2++ ) {
      switch ( chars_per_pixel ) {
        case 1:
          fprintf ( fp, "%c", xpm_chars[data[loop*image->width+loop2]] );
          break;
        case 2:
          fprintf ( fp, "%c%c",
            xpm_chars[data[loop*image->width+loop2]/strlen(xpm_chars)],
            xpm_chars[data[loop*image->width+loop2]%strlen(xpm_chars)] );
          break;
      }
    }
    fprintf ( fp, "\",\n" );
  }

  fprintf ( fp, "};\n" );

  /* free up allocated resources */
  free ( data );
  for ( loop = 0; loop < num_colors; loop++ ) {
    free ( colormap[loop] );
  }

  return ( INoError );
}





IError _IReadXPM ( fp, options, image_return )
FILE *fp;
IOptions options;
IImageP **image_return;
{
  return ( IFunctionNotImplemented );
}

