#!/usr/local/bin/perl
#
# h2html.pl
#
# Image library
#
# Description:
#	Create HTML documentation from a C include file.
#
# History:
#	19-Jul-99	Craig Knudsen	cknudsen@radix.net
#			Modified for nicer looking output.
#	29-May-96	Craig Knudsen	cknudsen@radix.net
#			Created
#
#######################################################################


sub print_function {
  $out{$name} = "<H3><A NAME=\"$name\">$name</A></H3>\n";
  $out{$name} .= "$description<P>\n"
    if ( defined ( $description ) );
  $out{$name} .= "Returns: <TT>$ret_type</TT><P>\n" .
    "Input Parameters:<BR>\n<UL>\n";
  for ( $i = 0; $i < $num_ivars; $i++ ) {
    $out{$name} .= "<LI><TT>$vars[$i]</TT>";
    $out{$name} .= " - $comments[$i]" if ( defined ( $comments[$i] ) );
    $out{$name} .= "\n";
  }
  $out{$name} .= "</UL><P>\n";
  if ( $i < $num_vars ) {
    $out{$name} .= "Output Parameters:<BR>\n<UL><P>\n";
    for ( ; $i < $num_vars; $i++ ) {
      $out{$name} .= "<LI><TT>$vars[$i]</TT>";
      $out{$name} .= " - $comments[$i]" if ( defined ( $comments[$i] ) );
      $out{$name} .= "\n";
    }
    $out{$name} .= "</UL>\n";
  }
}

while ( <> ) {
  chop;
  if ( /^([^\*]\S+)\s+(\S+)\s+\(/ ) {
    # start of a function.
    $name = $2;
    $ret_type = $1;
    if ( $name =~ /^\*/ ) {
      $name = $';
      $ret_type .= " *";
    }
    $name =~ s/^_//;
    $num_vars = 0;
    $num_ivars = 0;
  } elsif ( defined ( $name ) ) {
    if ( /^\s+(\S.*)\s+\/\*\s*(\S.*)\s*\*\// ) {
      $vars[$num_vars] = $1;
      $comments[$num_vars] = $2;
      if ( $comments[$num_vars] =~ /^out: / ) {
        $comments[$num_vars] =~ s/^out:\s*//;;
      } else {
        $num_ivars++;
      }
      $vars[$num_vars] =~ s/[\s,]+$//;
      $num_vars++;
    } elsif ( /^\);/ ) {
      &print_function;
      undef ( $name );
      undef ( @vars );
      undef ( @comments );
      undef ( $description );
    } elsif ( /\s+\/\*\s*(\S.*)\s*\*\// ) {
      $comments[$num_vars-1] .= " " . $1;
    }
  } elsif ( /Description:/ ) {
    $in_info = 1;
  } elsif ( /History:/ ) {
    $in_info = 0;
  } elsif ( $in_info ) {
    if ( /\*\s+(\S.*)$/ ) {
      $info .= " " if length ( $info );
      $info .= $1;
    }
  } elsif ( /^\*+\// ) {
    # end comment
  } elsif ( /^\*+\s*(\S.*)$/ ) {
    $description .= " " if ( length ( $description ) );
    $description .= $1;
  } elsif ( /^\/*/ ) {
    undef ( $description );
  }
}

print<<EOF;
<HTML>
<HEAD>
<TITLE>ILib API Documentation</TITLE>
</HEAD>
<BODY>
<H2>Ilib Image Library</H2>
$info
<H2>API Documentation</H2>
<UL>
EOF

foreach $name ( sort keys ( %out ) ) {
  print "<LI><A HREF=\"#$name\">$name</A>\n";
}

print "</UL>\n<HR>\n";

foreach $name ( sort keys ( %out ) ) {
  print "<P>\n" . $out{$name};
}

print "</BODY>\n</HTML>\n";

exit 0;
